# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import pydantic_v1


class StsTransferApi(pydantic_v1.BaseModel):
    aws_access_key_id: str = pydantic_v1.Field()
    """
    AWS access key.
    """

    aws_secret_access_key: str = pydantic_v1.Field()
    """
    AWS secret accesss key.
    """

    aws_session_token: typing.Optional[str] = None
    s3bucket: str = pydantic_v1.Field(alias="s3_bucket")
    """
    S3 bucket.
    """

    s3endpoint_url: typing.Optional[str] = pydantic_v1.Field(alias="s3_endpoint_url", default=None)
    s3key: str = pydantic_v1.Field(alias="s3_key")
    """
    S3 key.
    """

    s3url: str = pydantic_v1.Field(alias="s3_url")
    """
    S3 full URL.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



from octoai.uploading_asset_lake import AsyncUploadingAssetLakeClient, UploadingAssetLakeClient
from .base_client import BaseOctoAI, AsyncBaseOctoAI

import os
import typing
import httpx

from .asset_lake.client import AssetLakeClient, AsyncAssetLakeClient
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .environment import OctoAIEnvironment
from .fine_tuning.client import AsyncFineTuningClient, FineTuningClient
from .image_gen.client import AsyncImageGenClient, ImageGenClient
from .text_gen.client import AsyncTextGenClient, TextGenClient


class OctoAI(BaseOctoAI):
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propogate to these functions.

    Parameters:
        - environment: OctoAIEnvironment. The environment to use for requests from the client. from .environment import OctoAIEnvironment

                                          Defaults to OctoAIEnvironment.PRODUCTION

        - api_key: typing.Optional[typing.Union[str, typing.Callable[[], str]]].

        - timeout: typing.Optional[float]. The timeout to be used, in seconds, for requests by default the timeout is 60 seconds.

        - httpx_client: typing.Optional[httpx.Client]. The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.
    ---
    from octoai.client import OctoAI

    client = OctoAI(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        environment: OctoAIEnvironment = OctoAIEnvironment.PRODUCTION,
        api_key: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = os.getenv("OCTOAI_TOKEN"),
        timeout: typing.Optional[float] = 60,
        httpx_client: typing.Optional[httpx.Client] = None
    ):
        super().__init__(
            environment=environment,
            api_key=api_key,
            timeout=timeout,
            httpx_client=httpx_client
        )

        self.asset_lake = UploadingAssetLakeClient(client_wrapper=self._client_wrapper)


class AsyncOctoAI(AsyncBaseOctoAI):
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propogate to these functions.

    Parameters:
        - environment: OctoAIEnvironment. The environment to use for requests from the client. from .environment import OctoAIEnvironment

                                          Defaults to OctoAIEnvironment.PRODUCTION

        - api_key: typing.Optional[typing.Union[str, typing.Callable[[], str]]].

        - timeout: typing.Optional[float]. The timeout to be used, in seconds, for requests by default the timeout is 60 seconds.

        - httpx_client: typing.Optional[httpx.AsyncClient]. The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.
    ---
    from octoai.client import AsyncOctoAI

    client = AsyncOctoAI(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        environment: OctoAIEnvironment = OctoAIEnvironment.PRODUCTION,
        api_key: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = os.getenv("OCTOAI_TOKEN"),
        timeout: typing.Optional[float] = 60,
        httpx_client: typing.Optional[httpx.AsyncClient] = None
    ):
        super().__init__(
            environment=environment,
            api_key=api_key,
            timeout=timeout,
            httpx_client=httpx_client
        )

        self.asset_lake = AsyncUploadingAssetLakeClient(client_wrapper=self._client_wrapper)


# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import pydantic_v1
from .lora_tune_checkpoint import LoraTuneCheckpoint
from .lora_tune_file import LoraTuneFile


class LoraTune(pydantic_v1.BaseModel):
    """
    A LoRA Tune.
    """

    base_checkpoint: LoraTuneCheckpoint = pydantic_v1.Field()
    """
    The base checkpoint used for this LoRA Tune.
    """

    files: typing.List[LoraTuneFile] = pydantic_v1.Field()
    """
    The files used for this LoRA Tune.
    """

    resize_images: typing.Optional[bool] = None
    seed: typing.Optional[int] = None
    steps: int = pydantic_v1.Field()
    """
    The number of steps used for this LoRA Tune.
    """

    trigger_words: typing.List[str] = pydantic_v1.Field()
    """
    The trigger words used for this tune. As of now, only one trigger word is supported. `List` is used for future-proofing.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import json
import typing
import urllib.parse
from json.decoder import JSONDecodeError

import httpx_sse

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.remove_none_from_dict import remove_none_from_dict
from ..core.request_options import RequestOptions
from .errors.internal_server_error import InternalServerError
from .errors.unprocessable_entity_error import UnprocessableEntityError
from .types.chat_completion_chunk import ChatCompletionChunk
from .types.chat_completion_request_ext import ChatCompletionRequestExt
from .types.chat_completion_response import ChatCompletionResponse
from .types.chat_completion_response_format import ChatCompletionResponseFormat
from .types.chat_message import ChatMessage
from .types.completion_response import CompletionResponse
from .types.create_chat_completion_request_function_call import CreateChatCompletionRequestFunctionCall
from .types.create_chat_completion_stream_request_function_call import CreateChatCompletionStreamRequestFunctionCall
from .types.error_response import ErrorResponse
from .types.function import Function
from .types.http_validation_error import HttpValidationError
from .types.prompt import Prompt
from .types.stop import Stop

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TextGenClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create_chat_completion_stream(
        self,
        *,
        frequency_penalty: typing.Optional[float] = OMIT,
        function_call: typing.Optional[CreateChatCompletionStreamRequestFunctionCall] = OMIT,
        functions: typing.Optional[typing.Sequence[Function]] = OMIT,
        ignore_eos: typing.Optional[bool] = OMIT,
        logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]] = OMIT,
        max_tokens: typing.Optional[float] = OMIT,
        messages: typing.Sequence[ChatMessage],
        model: str,
        n: typing.Optional[int] = OMIT,
        octoai: typing.Optional[ChatCompletionRequestExt] = OMIT,
        presence_penalty: typing.Optional[float] = OMIT,
        repetition_penalty: typing.Optional[float] = OMIT,
        response_format: typing.Optional[ChatCompletionResponseFormat] = OMIT,
        stop: typing.Optional[Stop] = OMIT,
        temperature: typing.Optional[float] = OMIT,
        top_p: typing.Optional[float] = OMIT,
        user: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[ChatCompletionChunk]:
        """
        An OpenAI compatible Chat Completion API.

        Parameters:
            - frequency_penalty: typing.Optional[float].

            - function_call: typing.Optional[CreateChatCompletionStreamRequestFunctionCall].

            - functions: typing.Optional[typing.Sequence[Function]].

            - ignore_eos: typing.Optional[bool].

            - logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]].

            - max_tokens: typing.Optional[float].

            - messages: typing.Sequence[ChatMessage]. A list of messages comprising the conversation so far.

            - model: str. The identifier of the model to use.Can be a shared tenancy or custom model identifier.

            - n: typing.Optional[int].

            - octoai: typing.Optional[ChatCompletionRequestExt].

            - presence_penalty: typing.Optional[float].

            - repetition_penalty: typing.Optional[float].

            - response_format: typing.Optional[ChatCompletionResponseFormat].

            - stop: typing.Optional[Stop].

            - temperature: typing.Optional[float].

            - top_p: typing.Optional[float].

            - user: typing.Optional[str].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import OctoAI
        from octoai.text_gen import (
            ChatCompletionRequestExt,
            ChatCompletionRequestExtVllm,
            ChatCompletionResponseFormat,
            ChatFnCall,
            ChatMessage,
            Function,
        )

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.text_gen.create_chat_completion_stream(
            frequency_penalty=1.1,
            function_call="string",
            functions=[
                Function(
                    description="string",
                    name="string",
                    parameters={"key": "value"},
                )
            ],
            ignore_eos=True,
            logit_bias={"string": {"key": "value"}},
            max_tokens=1.1,
            messages=[
                ChatMessage(
                    content="string",
                    function_call=ChatFnCall(
                        arguments="string",
                        name="string",
                    ),
                    role="string",
                )
            ],
            model="string",
            n=1,
            octoai=ChatCompletionRequestExt(
                loras=["string"],
                vllm=ChatCompletionRequestExtVllm(),
            ),
            presence_penalty=1.1,
            repetition_penalty=1.1,
            response_format=ChatCompletionResponseFormat(
                schema={"string": {"key": "value"}},
                type="string",
            ),
            stop="string",
            temperature=1.1,
            top_p=1.1,
            user="string",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"messages": messages, "model": model, "stream": True}
        if frequency_penalty is not OMIT:
            _request["frequency_penalty"] = frequency_penalty
        if function_call is not OMIT:
            _request["function_call"] = function_call
        if functions is not OMIT:
            _request["functions"] = functions
        if ignore_eos is not OMIT:
            _request["ignore_eos"] = ignore_eos
        if logit_bias is not OMIT:
            _request["logit_bias"] = logit_bias
        if max_tokens is not OMIT:
            _request["max_tokens"] = max_tokens
        if n is not OMIT:
            _request["n"] = n
        if octoai is not OMIT:
            _request["octoai"] = octoai
        if presence_penalty is not OMIT:
            _request["presence_penalty"] = presence_penalty
        if repetition_penalty is not OMIT:
            _request["repetition_penalty"] = repetition_penalty
        if response_format is not OMIT:
            _request["response_format"] = response_format
        if stop is not OMIT:
            _request["stop"] = stop
        if temperature is not OMIT:
            _request["temperature"] = temperature
        if top_p is not OMIT:
            _request["top_p"] = top_p
        if user is not OMIT:
            _request["user"] = user
        with self._client_wrapper.httpx_client.stream(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_environment().text_gen}/", "v1/chat/completions"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        ) as _response:
            if 200 <= _response.status_code < 300:
                _event_source = httpx_sse.EventSource(_response)
                for _sse in _event_source.iter_sse():
                    yield pydantic_v1.parse_obj_as(ChatCompletionChunk, json.loads(_sse.data))  # type: ignore
                return
            _response.read()
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            if _response.status_code == 500:
                raise InternalServerError(pydantic_v1.parse_obj_as(ErrorResponse, _response.json()))  # type: ignore
            try:
                _response_json = _response.json()
            except JSONDecodeError:
                raise ApiError(status_code=_response.status_code, body=_response.text)
            raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_chat_completion(
        self,
        *,
        frequency_penalty: typing.Optional[float] = OMIT,
        function_call: typing.Optional[CreateChatCompletionRequestFunctionCall] = OMIT,
        functions: typing.Optional[typing.Sequence[Function]] = OMIT,
        ignore_eos: typing.Optional[bool] = OMIT,
        logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]] = OMIT,
        max_tokens: typing.Optional[float] = OMIT,
        messages: typing.Sequence[ChatMessage],
        model: str,
        n: typing.Optional[int] = OMIT,
        octoai: typing.Optional[ChatCompletionRequestExt] = OMIT,
        presence_penalty: typing.Optional[float] = OMIT,
        repetition_penalty: typing.Optional[float] = OMIT,
        response_format: typing.Optional[ChatCompletionResponseFormat] = OMIT,
        stop: typing.Optional[Stop] = OMIT,
        temperature: typing.Optional[float] = OMIT,
        top_p: typing.Optional[float] = OMIT,
        user: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ChatCompletionResponse:
        """
        An OpenAI compatible Chat Completion API.

        Parameters:
            - frequency_penalty: typing.Optional[float].

            - function_call: typing.Optional[CreateChatCompletionRequestFunctionCall].

            - functions: typing.Optional[typing.Sequence[Function]].

            - ignore_eos: typing.Optional[bool].

            - logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]].

            - max_tokens: typing.Optional[float].

            - messages: typing.Sequence[ChatMessage]. A list of messages comprising the conversation so far.

            - model: str. The identifier of the model to use.Can be a shared tenancy or custom model identifier.

            - n: typing.Optional[int].

            - octoai: typing.Optional[ChatCompletionRequestExt].

            - presence_penalty: typing.Optional[float].

            - repetition_penalty: typing.Optional[float].

            - response_format: typing.Optional[ChatCompletionResponseFormat].

            - stop: typing.Optional[Stop].

            - temperature: typing.Optional[float].

            - top_p: typing.Optional[float].

            - user: typing.Optional[str].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import OctoAI
        from octoai.text_gen import ChatMessage

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.text_gen.create_chat_completion(
            messages=[
                ChatMessage(
                    role="role",
                )
            ],
            model="model",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"messages": messages, "model": model, "stream": False}
        if frequency_penalty is not OMIT:
            _request["frequency_penalty"] = frequency_penalty
        if function_call is not OMIT:
            _request["function_call"] = function_call
        if functions is not OMIT:
            _request["functions"] = functions
        if ignore_eos is not OMIT:
            _request["ignore_eos"] = ignore_eos
        if logit_bias is not OMIT:
            _request["logit_bias"] = logit_bias
        if max_tokens is not OMIT:
            _request["max_tokens"] = max_tokens
        if n is not OMIT:
            _request["n"] = n
        if octoai is not OMIT:
            _request["octoai"] = octoai
        if presence_penalty is not OMIT:
            _request["presence_penalty"] = presence_penalty
        if repetition_penalty is not OMIT:
            _request["repetition_penalty"] = repetition_penalty
        if response_format is not OMIT:
            _request["response_format"] = response_format
        if stop is not OMIT:
            _request["stop"] = stop
        if temperature is not OMIT:
            _request["temperature"] = temperature
        if top_p is not OMIT:
            _request["top_p"] = top_p
        if user is not OMIT:
            _request["user"] = user
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_environment().text_gen}/", "v1/chat/completions"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ChatCompletionResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        if _response.status_code == 500:
            raise InternalServerError(pydantic_v1.parse_obj_as(ErrorResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_completion_stream(
        self,
        *,
        best_of: typing.Optional[int] = OMIT,
        echo: typing.Optional[bool] = OMIT,
        frequency_penalty: typing.Optional[float] = OMIT,
        logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]] = OMIT,
        logprobs: typing.Optional[int] = OMIT,
        max_tokens: typing.Optional[int] = OMIT,
        model: str,
        n: typing.Optional[int] = OMIT,
        presence_penalty: typing.Optional[float] = OMIT,
        prompt: Prompt,
        repetition_penalty: typing.Optional[float] = OMIT,
        seed: typing.Optional[int] = OMIT,
        stop: typing.Optional[Stop] = OMIT,
        suffix: typing.Optional[str] = OMIT,
        temperature: typing.Optional[float] = OMIT,
        top_p: typing.Optional[float] = OMIT,
        user: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[CompletionResponse]:
        """
        An OpenAI compatible Completion API.

        Parameters:
            - best_of: typing.Optional[int].

            - echo: typing.Optional[bool].

            - frequency_penalty: typing.Optional[float].

            - logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]].

            - logprobs: typing.Optional[int].

            - max_tokens: typing.Optional[int].

            - model: str. Model name to use for completion.

            - n: typing.Optional[int].

            - presence_penalty: typing.Optional[float].

            - prompt: Prompt. The prompt to generate completions from.

            - repetition_penalty: typing.Optional[float].

            - seed: typing.Optional[int].

            - stop: typing.Optional[Stop]. Strings that stop the generation when they are generated.

            - suffix: typing.Optional[str].

            - temperature: typing.Optional[float].

            - top_p: typing.Optional[float].

            - user: typing.Optional[str].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import OctoAI

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.text_gen.create_completion_stream(
            best_of=1,
            echo=True,
            frequency_penalty=1.1,
            logit_bias={"string": {"key": "value"}},
            logprobs=1,
            max_tokens=1,
            model="string",
            n=1,
            presence_penalty=1.1,
            prompt="string",
            repetition_penalty=1.1,
            seed=1,
            stop="string",
            suffix="string",
            temperature=1.1,
            top_p=1.1,
            user="string",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"model": model, "prompt": prompt, "stream": True}
        if best_of is not OMIT:
            _request["best_of"] = best_of
        if echo is not OMIT:
            _request["echo"] = echo
        if frequency_penalty is not OMIT:
            _request["frequency_penalty"] = frequency_penalty
        if logit_bias is not OMIT:
            _request["logit_bias"] = logit_bias
        if logprobs is not OMIT:
            _request["logprobs"] = logprobs
        if max_tokens is not OMIT:
            _request["max_tokens"] = max_tokens
        if n is not OMIT:
            _request["n"] = n
        if presence_penalty is not OMIT:
            _request["presence_penalty"] = presence_penalty
        if repetition_penalty is not OMIT:
            _request["repetition_penalty"] = repetition_penalty
        if seed is not OMIT:
            _request["seed"] = seed
        if stop is not OMIT:
            _request["stop"] = stop
        if suffix is not OMIT:
            _request["suffix"] = suffix
        if temperature is not OMIT:
            _request["temperature"] = temperature
        if top_p is not OMIT:
            _request["top_p"] = top_p
        if user is not OMIT:
            _request["user"] = user
        with self._client_wrapper.httpx_client.stream(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_environment().text_gen}/", "v1/completions"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        ) as _response:
            if 200 <= _response.status_code < 300:
                _event_source = httpx_sse.EventSource(_response)
                for _sse in _event_source.iter_sse():
                    yield pydantic_v1.parse_obj_as(CompletionResponse, json.loads(_sse.data))  # type: ignore
                return
            _response.read()
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            try:
                _response_json = _response.json()
            except JSONDecodeError:
                raise ApiError(status_code=_response.status_code, body=_response.text)
            raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_completion(
        self,
        *,
        best_of: typing.Optional[int] = OMIT,
        echo: typing.Optional[bool] = OMIT,
        frequency_penalty: typing.Optional[float] = OMIT,
        logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]] = OMIT,
        logprobs: typing.Optional[int] = OMIT,
        max_tokens: typing.Optional[int] = OMIT,
        model: str,
        n: typing.Optional[int] = OMIT,
        presence_penalty: typing.Optional[float] = OMIT,
        prompt: Prompt,
        repetition_penalty: typing.Optional[float] = OMIT,
        seed: typing.Optional[int] = OMIT,
        stop: typing.Optional[Stop] = OMIT,
        suffix: typing.Optional[str] = OMIT,
        temperature: typing.Optional[float] = OMIT,
        top_p: typing.Optional[float] = OMIT,
        user: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CompletionResponse:
        """
        An OpenAI compatible Completion API.

        Parameters:
            - best_of: typing.Optional[int].

            - echo: typing.Optional[bool].

            - frequency_penalty: typing.Optional[float].

            - logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]].

            - logprobs: typing.Optional[int].

            - max_tokens: typing.Optional[int].

            - model: str. Model name to use for completion.

            - n: typing.Optional[int].

            - presence_penalty: typing.Optional[float].

            - prompt: Prompt. The prompt to generate completions from.

            - repetition_penalty: typing.Optional[float].

            - seed: typing.Optional[int].

            - stop: typing.Optional[Stop]. Strings that stop the generation when they are generated.

            - suffix: typing.Optional[str].

            - temperature: typing.Optional[float].

            - top_p: typing.Optional[float].

            - user: typing.Optional[str].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import OctoAI

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.text_gen.create_completion(
            model="model",
            prompt="prompt",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"model": model, "prompt": prompt, "stream": False}
        if best_of is not OMIT:
            _request["best_of"] = best_of
        if echo is not OMIT:
            _request["echo"] = echo
        if frequency_penalty is not OMIT:
            _request["frequency_penalty"] = frequency_penalty
        if logit_bias is not OMIT:
            _request["logit_bias"] = logit_bias
        if logprobs is not OMIT:
            _request["logprobs"] = logprobs
        if max_tokens is not OMIT:
            _request["max_tokens"] = max_tokens
        if n is not OMIT:
            _request["n"] = n
        if presence_penalty is not OMIT:
            _request["presence_penalty"] = presence_penalty
        if repetition_penalty is not OMIT:
            _request["repetition_penalty"] = repetition_penalty
        if seed is not OMIT:
            _request["seed"] = seed
        if stop is not OMIT:
            _request["stop"] = stop
        if suffix is not OMIT:
            _request["suffix"] = suffix
        if temperature is not OMIT:
            _request["temperature"] = temperature
        if top_p is not OMIT:
            _request["top_p"] = top_p
        if user is not OMIT:
            _request["user"] = user
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_environment().text_gen}/", "v1/completions"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(CompletionResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncTextGenClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create_chat_completion_stream(
        self,
        *,
        frequency_penalty: typing.Optional[float] = OMIT,
        function_call: typing.Optional[CreateChatCompletionStreamRequestFunctionCall] = OMIT,
        functions: typing.Optional[typing.Sequence[Function]] = OMIT,
        ignore_eos: typing.Optional[bool] = OMIT,
        logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]] = OMIT,
        max_tokens: typing.Optional[float] = OMIT,
        messages: typing.Sequence[ChatMessage],
        model: str,
        n: typing.Optional[int] = OMIT,
        octoai: typing.Optional[ChatCompletionRequestExt] = OMIT,
        presence_penalty: typing.Optional[float] = OMIT,
        repetition_penalty: typing.Optional[float] = OMIT,
        response_format: typing.Optional[ChatCompletionResponseFormat] = OMIT,
        stop: typing.Optional[Stop] = OMIT,
        temperature: typing.Optional[float] = OMIT,
        top_p: typing.Optional[float] = OMIT,
        user: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[ChatCompletionChunk]:
        """
        An OpenAI compatible Chat Completion API.

        Parameters:
            - frequency_penalty: typing.Optional[float].

            - function_call: typing.Optional[CreateChatCompletionStreamRequestFunctionCall].

            - functions: typing.Optional[typing.Sequence[Function]].

            - ignore_eos: typing.Optional[bool].

            - logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]].

            - max_tokens: typing.Optional[float].

            - messages: typing.Sequence[ChatMessage]. A list of messages comprising the conversation so far.

            - model: str. The identifier of the model to use.Can be a shared tenancy or custom model identifier.

            - n: typing.Optional[int].

            - octoai: typing.Optional[ChatCompletionRequestExt].

            - presence_penalty: typing.Optional[float].

            - repetition_penalty: typing.Optional[float].

            - response_format: typing.Optional[ChatCompletionResponseFormat].

            - stop: typing.Optional[Stop].

            - temperature: typing.Optional[float].

            - top_p: typing.Optional[float].

            - user: typing.Optional[str].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import AsyncOctoAI
        from octoai.text_gen import (
            ChatCompletionRequestExt,
            ChatCompletionRequestExtVllm,
            ChatCompletionResponseFormat,
            ChatFnCall,
            ChatMessage,
            Function,
        )

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.text_gen.create_chat_completion_stream(
            frequency_penalty=1.1,
            function_call="string",
            functions=[
                Function(
                    description="string",
                    name="string",
                    parameters={"key": "value"},
                )
            ],
            ignore_eos=True,
            logit_bias={"string": {"key": "value"}},
            max_tokens=1.1,
            messages=[
                ChatMessage(
                    content="string",
                    function_call=ChatFnCall(
                        arguments="string",
                        name="string",
                    ),
                    role="string",
                )
            ],
            model="string",
            n=1,
            octoai=ChatCompletionRequestExt(
                loras=["string"],
                vllm=ChatCompletionRequestExtVllm(),
            ),
            presence_penalty=1.1,
            repetition_penalty=1.1,
            response_format=ChatCompletionResponseFormat(
                schema={"string": {"key": "value"}},
                type="string",
            ),
            stop="string",
            temperature=1.1,
            top_p=1.1,
            user="string",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"messages": messages, "model": model, "stream": True}
        if frequency_penalty is not OMIT:
            _request["frequency_penalty"] = frequency_penalty
        if function_call is not OMIT:
            _request["function_call"] = function_call
        if functions is not OMIT:
            _request["functions"] = functions
        if ignore_eos is not OMIT:
            _request["ignore_eos"] = ignore_eos
        if logit_bias is not OMIT:
            _request["logit_bias"] = logit_bias
        if max_tokens is not OMIT:
            _request["max_tokens"] = max_tokens
        if n is not OMIT:
            _request["n"] = n
        if octoai is not OMIT:
            _request["octoai"] = octoai
        if presence_penalty is not OMIT:
            _request["presence_penalty"] = presence_penalty
        if repetition_penalty is not OMIT:
            _request["repetition_penalty"] = repetition_penalty
        if response_format is not OMIT:
            _request["response_format"] = response_format
        if stop is not OMIT:
            _request["stop"] = stop
        if temperature is not OMIT:
            _request["temperature"] = temperature
        if top_p is not OMIT:
            _request["top_p"] = top_p
        if user is not OMIT:
            _request["user"] = user
        async with self._client_wrapper.httpx_client.stream(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_environment().text_gen}/", "v1/chat/completions"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        ) as _response:
            if 200 <= _response.status_code < 300:
                _event_source = httpx_sse.EventSource(_response)
                async for _sse in _event_source.aiter_sse():
                    yield pydantic_v1.parse_obj_as(ChatCompletionChunk, json.loads(_sse.data))  # type: ignore
                return
            await _response.aread()
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            if _response.status_code == 500:
                raise InternalServerError(pydantic_v1.parse_obj_as(ErrorResponse, _response.json()))  # type: ignore
            try:
                _response_json = _response.json()
            except JSONDecodeError:
                raise ApiError(status_code=_response.status_code, body=_response.text)
            raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_chat_completion(
        self,
        *,
        frequency_penalty: typing.Optional[float] = OMIT,
        function_call: typing.Optional[CreateChatCompletionRequestFunctionCall] = OMIT,
        functions: typing.Optional[typing.Sequence[Function]] = OMIT,
        ignore_eos: typing.Optional[bool] = OMIT,
        logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]] = OMIT,
        max_tokens: typing.Optional[float] = OMIT,
        messages: typing.Sequence[ChatMessage],
        model: str,
        n: typing.Optional[int] = OMIT,
        octoai: typing.Optional[ChatCompletionRequestExt] = OMIT,
        presence_penalty: typing.Optional[float] = OMIT,
        repetition_penalty: typing.Optional[float] = OMIT,
        response_format: typing.Optional[ChatCompletionResponseFormat] = OMIT,
        stop: typing.Optional[Stop] = OMIT,
        temperature: typing.Optional[float] = OMIT,
        top_p: typing.Optional[float] = OMIT,
        user: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ChatCompletionResponse:
        """
        An OpenAI compatible Chat Completion API.

        Parameters:
            - frequency_penalty: typing.Optional[float].

            - function_call: typing.Optional[CreateChatCompletionRequestFunctionCall].

            - functions: typing.Optional[typing.Sequence[Function]].

            - ignore_eos: typing.Optional[bool].

            - logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]].

            - max_tokens: typing.Optional[float].

            - messages: typing.Sequence[ChatMessage]. A list of messages comprising the conversation so far.

            - model: str. The identifier of the model to use.Can be a shared tenancy or custom model identifier.

            - n: typing.Optional[int].

            - octoai: typing.Optional[ChatCompletionRequestExt].

            - presence_penalty: typing.Optional[float].

            - repetition_penalty: typing.Optional[float].

            - response_format: typing.Optional[ChatCompletionResponseFormat].

            - stop: typing.Optional[Stop].

            - temperature: typing.Optional[float].

            - top_p: typing.Optional[float].

            - user: typing.Optional[str].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import AsyncOctoAI
        from octoai.text_gen import ChatMessage

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.text_gen.create_chat_completion(
            messages=[
                ChatMessage(
                    role="role",
                )
            ],
            model="model",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"messages": messages, "model": model, "stream": False}
        if frequency_penalty is not OMIT:
            _request["frequency_penalty"] = frequency_penalty
        if function_call is not OMIT:
            _request["function_call"] = function_call
        if functions is not OMIT:
            _request["functions"] = functions
        if ignore_eos is not OMIT:
            _request["ignore_eos"] = ignore_eos
        if logit_bias is not OMIT:
            _request["logit_bias"] = logit_bias
        if max_tokens is not OMIT:
            _request["max_tokens"] = max_tokens
        if n is not OMIT:
            _request["n"] = n
        if octoai is not OMIT:
            _request["octoai"] = octoai
        if presence_penalty is not OMIT:
            _request["presence_penalty"] = presence_penalty
        if repetition_penalty is not OMIT:
            _request["repetition_penalty"] = repetition_penalty
        if response_format is not OMIT:
            _request["response_format"] = response_format
        if stop is not OMIT:
            _request["stop"] = stop
        if temperature is not OMIT:
            _request["temperature"] = temperature
        if top_p is not OMIT:
            _request["top_p"] = top_p
        if user is not OMIT:
            _request["user"] = user
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_environment().text_gen}/", "v1/chat/completions"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ChatCompletionResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        if _response.status_code == 500:
            raise InternalServerError(pydantic_v1.parse_obj_as(ErrorResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_completion_stream(
        self,
        *,
        best_of: typing.Optional[int] = OMIT,
        echo: typing.Optional[bool] = OMIT,
        frequency_penalty: typing.Optional[float] = OMIT,
        logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]] = OMIT,
        logprobs: typing.Optional[int] = OMIT,
        max_tokens: typing.Optional[int] = OMIT,
        model: str,
        n: typing.Optional[int] = OMIT,
        presence_penalty: typing.Optional[float] = OMIT,
        prompt: Prompt,
        repetition_penalty: typing.Optional[float] = OMIT,
        seed: typing.Optional[int] = OMIT,
        stop: typing.Optional[Stop] = OMIT,
        suffix: typing.Optional[str] = OMIT,
        temperature: typing.Optional[float] = OMIT,
        top_p: typing.Optional[float] = OMIT,
        user: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[CompletionResponse]:
        """
        An OpenAI compatible Completion API.

        Parameters:
            - best_of: typing.Optional[int].

            - echo: typing.Optional[bool].

            - frequency_penalty: typing.Optional[float].

            - logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]].

            - logprobs: typing.Optional[int].

            - max_tokens: typing.Optional[int].

            - model: str. Model name to use for completion.

            - n: typing.Optional[int].

            - presence_penalty: typing.Optional[float].

            - prompt: Prompt. The prompt to generate completions from.

            - repetition_penalty: typing.Optional[float].

            - seed: typing.Optional[int].

            - stop: typing.Optional[Stop]. Strings that stop the generation when they are generated.

            - suffix: typing.Optional[str].

            - temperature: typing.Optional[float].

            - top_p: typing.Optional[float].

            - user: typing.Optional[str].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import AsyncOctoAI

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.text_gen.create_completion_stream(
            best_of=1,
            echo=True,
            frequency_penalty=1.1,
            logit_bias={"string": {"key": "value"}},
            logprobs=1,
            max_tokens=1,
            model="string",
            n=1,
            presence_penalty=1.1,
            prompt="string",
            repetition_penalty=1.1,
            seed=1,
            stop="string",
            suffix="string",
            temperature=1.1,
            top_p=1.1,
            user="string",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"model": model, "prompt": prompt, "stream": True}
        if best_of is not OMIT:
            _request["best_of"] = best_of
        if echo is not OMIT:
            _request["echo"] = echo
        if frequency_penalty is not OMIT:
            _request["frequency_penalty"] = frequency_penalty
        if logit_bias is not OMIT:
            _request["logit_bias"] = logit_bias
        if logprobs is not OMIT:
            _request["logprobs"] = logprobs
        if max_tokens is not OMIT:
            _request["max_tokens"] = max_tokens
        if n is not OMIT:
            _request["n"] = n
        if presence_penalty is not OMIT:
            _request["presence_penalty"] = presence_penalty
        if repetition_penalty is not OMIT:
            _request["repetition_penalty"] = repetition_penalty
        if seed is not OMIT:
            _request["seed"] = seed
        if stop is not OMIT:
            _request["stop"] = stop
        if suffix is not OMIT:
            _request["suffix"] = suffix
        if temperature is not OMIT:
            _request["temperature"] = temperature
        if top_p is not OMIT:
            _request["top_p"] = top_p
        if user is not OMIT:
            _request["user"] = user
        async with self._client_wrapper.httpx_client.stream(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_environment().text_gen}/", "v1/completions"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        ) as _response:
            if 200 <= _response.status_code < 300:
                _event_source = httpx_sse.EventSource(_response)
                async for _sse in _event_source.aiter_sse():
                    yield pydantic_v1.parse_obj_as(CompletionResponse, json.loads(_sse.data))  # type: ignore
                return
            await _response.aread()
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
                )
            try:
                _response_json = _response.json()
            except JSONDecodeError:
                raise ApiError(status_code=_response.status_code, body=_response.text)
            raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_completion(
        self,
        *,
        best_of: typing.Optional[int] = OMIT,
        echo: typing.Optional[bool] = OMIT,
        frequency_penalty: typing.Optional[float] = OMIT,
        logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]] = OMIT,
        logprobs: typing.Optional[int] = OMIT,
        max_tokens: typing.Optional[int] = OMIT,
        model: str,
        n: typing.Optional[int] = OMIT,
        presence_penalty: typing.Optional[float] = OMIT,
        prompt: Prompt,
        repetition_penalty: typing.Optional[float] = OMIT,
        seed: typing.Optional[int] = OMIT,
        stop: typing.Optional[Stop] = OMIT,
        suffix: typing.Optional[str] = OMIT,
        temperature: typing.Optional[float] = OMIT,
        top_p: typing.Optional[float] = OMIT,
        user: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CompletionResponse:
        """
        An OpenAI compatible Completion API.

        Parameters:
            - best_of: typing.Optional[int].

            - echo: typing.Optional[bool].

            - frequency_penalty: typing.Optional[float].

            - logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]].

            - logprobs: typing.Optional[int].

            - max_tokens: typing.Optional[int].

            - model: str. Model name to use for completion.

            - n: typing.Optional[int].

            - presence_penalty: typing.Optional[float].

            - prompt: Prompt. The prompt to generate completions from.

            - repetition_penalty: typing.Optional[float].

            - seed: typing.Optional[int].

            - stop: typing.Optional[Stop]. Strings that stop the generation when they are generated.

            - suffix: typing.Optional[str].

            - temperature: typing.Optional[float].

            - top_p: typing.Optional[float].

            - user: typing.Optional[str].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import AsyncOctoAI

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.text_gen.create_completion(
            model="model",
            prompt="prompt",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"model": model, "prompt": prompt, "stream": False}
        if best_of is not OMIT:
            _request["best_of"] = best_of
        if echo is not OMIT:
            _request["echo"] = echo
        if frequency_penalty is not OMIT:
            _request["frequency_penalty"] = frequency_penalty
        if logit_bias is not OMIT:
            _request["logit_bias"] = logit_bias
        if logprobs is not OMIT:
            _request["logprobs"] = logprobs
        if max_tokens is not OMIT:
            _request["max_tokens"] = max_tokens
        if n is not OMIT:
            _request["n"] = n
        if presence_penalty is not OMIT:
            _request["presence_penalty"] = presence_penalty
        if repetition_penalty is not OMIT:
            _request["repetition_penalty"] = repetition_penalty
        if seed is not OMIT:
            _request["seed"] = seed
        if stop is not OMIT:
            _request["stop"] = stop
        if suffix is not OMIT:
            _request["suffix"] = suffix
        if temperature is not OMIT:
            _request["temperature"] = temperature
        if top_p is not OMIT:
            _request["top_p"] = top_p
        if user is not OMIT:
            _request["user"] = user
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_environment().text_gen}/", "v1/completions"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(CompletionResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

BaseEngine = typing.Union[
    typing.Literal[
        "text/llama-2-7b",
        "text/llama-2-13b-chat",
        "text/llama-2-70b-chat",
        "text/codellama-7b-instruct",
        "text/codellama-13b-instruct",
        "text/codellama-34b-instruct",
        "text/mistral-7b-instruct",
        "text/mixtral-8x7b-instruct",
        "image/controlnet-sd15",
        "image/controlnet-sdxl",
        "image/stable-diffusion-v1-5",
        "image/stable-diffusion-xl-v1-0",
        "image/stable-diffusion-ssd",
        "audio/tortoise-tts",
        "audio/rvc",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import os
import typing

import httpx

from .asset_lake.client import AssetLakeClient, AsyncAssetLakeClient
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .environment import OctoAIEnvironment
from .fine_tuning.client import AsyncFineTuningClient, FineTuningClient
from .image_gen.client import AsyncImageGenClient, ImageGenClient
from .text_gen.client import AsyncTextGenClient, TextGenClient


class BaseOctoAI:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propogate to these functions.

    Parameters:
        - environment: OctoAIEnvironment. The environment to use for requests from the client. from .environment import OctoAIEnvironment

                                          Defaults to OctoAIEnvironment.PRODUCTION

        - api_key: typing.Optional[typing.Union[str, typing.Callable[[], str]]].

        - timeout: typing.Optional[float]. The timeout to be used, in seconds, for requests by default the timeout is 60 seconds, unless a custom httpx client is used, in which case a default is not set.

        - follow_redirects: typing.Optional[bool]. Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

        - httpx_client: typing.Optional[httpx.Client]. The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.
    ---
    from octoai.client import OctoAI

    client = OctoAI(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        environment: OctoAIEnvironment = OctoAIEnvironment.PRODUCTION,
        api_key: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = os.getenv("OCTOAI_TOKEN"),
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = None,
        httpx_client: typing.Optional[httpx.Client] = None
    ):
        _defaulted_timeout = timeout if timeout is not None else 60 if httpx_client is None else None
        self._client_wrapper = SyncClientWrapper(
            environment=environment,
            api_key=api_key,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.asset_lake = AssetLakeClient(client_wrapper=self._client_wrapper)
        self.fine_tuning = FineTuningClient(client_wrapper=self._client_wrapper)
        self.image_gen = ImageGenClient(client_wrapper=self._client_wrapper)
        self.text_gen = TextGenClient(client_wrapper=self._client_wrapper)


class AsyncBaseOctoAI:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propogate to these functions.

    Parameters:
        - environment: OctoAIEnvironment. The environment to use for requests from the client. from .environment import OctoAIEnvironment

                                          Defaults to OctoAIEnvironment.PRODUCTION

        - api_key: typing.Optional[typing.Union[str, typing.Callable[[], str]]].

        - timeout: typing.Optional[float]. The timeout to be used, in seconds, for requests by default the timeout is 60 seconds, unless a custom httpx client is used, in which case a default is not set.

        - follow_redirects: typing.Optional[bool]. Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

        - httpx_client: typing.Optional[httpx.AsyncClient]. The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.
    ---
    from octoai.client import AsyncOctoAI

    client = AsyncOctoAI(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        environment: OctoAIEnvironment = OctoAIEnvironment.PRODUCTION,
        api_key: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = os.getenv("OCTOAI_TOKEN"),
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = None,
        httpx_client: typing.Optional[httpx.AsyncClient] = None
    ):
        _defaulted_timeout = timeout if timeout is not None else 60 if httpx_client is None else None
        self._client_wrapper = AsyncClientWrapper(
            environment=environment,
            api_key=api_key,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.asset_lake = AsyncAssetLakeClient(client_wrapper=self._client_wrapper)
        self.fine_tuning = AsyncFineTuningClient(client_wrapper=self._client_wrapper)
        self.image_gen = AsyncImageGenClient(client_wrapper=self._client_wrapper)
        self.text_gen = AsyncTextGenClient(client_wrapper=self._client_wrapper)



# This file was auto-generated by Fern from our API Definition.

from .asset import Asset
from .asset_type import AssetType
from .base_engine import BaseEngine
from .base_engine_type import BaseEngineType
from .checkpoint_data import CheckpointData
from .complete_asset_upload_response import CompleteAssetUploadResponse
from .create_asset_response import CreateAssetResponse
from .create_asset_response_transfer_api import (
    CreateAssetResponseTransferApi,
    CreateAssetResponseTransferApi_None,
    CreateAssetResponseTransferApi_PresignedUrl,
    CreateAssetResponseTransferApi_Sts,
)
from .data import Data, Data_Checkpoint, Data_File, Data_Latent, Data_Lora, Data_TextualInversion, Data_Vae
from .data_type import DataType
from .delete_asset_response import DeleteAssetResponse
from .file_data import FileData
from .file_extension import FileExtension
from .file_format import FileFormat
from .file_structure import FileStructure
from .http_validation_error import HttpValidationError
from .latent_data import LatentData
from .list_assets_response import ListAssetsResponse
from .lora_data import LoraData
from .no_transfer_api import NoTransferApi
from .presigned_url_transfer_api import PresignedUrlTransferApi
from .retrieve_asset_response import RetrieveAssetResponse
from .retrieve_asset_response_transfer_api import (
    RetrieveAssetResponseTransferApi,
    RetrieveAssetResponseTransferApi_None,
    RetrieveAssetResponseTransferApi_PresignedUrl,
    RetrieveAssetResponseTransferApi_Sts,
)
from .status import Status
from .sts_transfer_api import StsTransferApi
from .textual_inversion_data import TextualInversionData
from .transfer_api_type import TransferApiType
from .vae_data import VaeData
from .validation_error import ValidationError
from .validation_error_loc_item import ValidationErrorLocItem

__all__ = [
    "Asset",
    "AssetType",
    "BaseEngine",
    "BaseEngineType",
    "CheckpointData",
    "CompleteAssetUploadResponse",
    "CreateAssetResponse",
    "CreateAssetResponseTransferApi",
    "CreateAssetResponseTransferApi_None",
    "CreateAssetResponseTransferApi_PresignedUrl",
    "CreateAssetResponseTransferApi_Sts",
    "Data",
    "DataType",
    "Data_Checkpoint",
    "Data_File",
    "Data_Latent",
    "Data_Lora",
    "Data_TextualInversion",
    "Data_Vae",
    "DeleteAssetResponse",
    "FileData",
    "FileExtension",
    "FileFormat",
    "FileStructure",
    "HttpValidationError",
    "LatentData",
    "ListAssetsResponse",
    "LoraData",
    "NoTransferApi",
    "PresignedUrlTransferApi",
    "RetrieveAssetResponse",
    "RetrieveAssetResponseTransferApi",
    "RetrieveAssetResponseTransferApi_None",
    "RetrieveAssetResponseTransferApi_PresignedUrl",
    "RetrieveAssetResponseTransferApi_Sts",
    "Status",
    "StsTransferApi",
    "TextualInversionData",
    "TransferApiType",
    "VaeData",
    "ValidationError",
    "ValidationErrorLocItem",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .no_transfer_api import NoTransferApi
from .presigned_url_transfer_api import PresignedUrlTransferApi
from .sts_transfer_api import StsTransferApi


class CreateAssetResponseTransferApi_None(NoTransferApi):
    type: typing.Literal["none"] = "none"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class CreateAssetResponseTransferApi_PresignedUrl(PresignedUrlTransferApi):
    type: typing.Literal["presigned-url"] = "presigned-url"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class CreateAssetResponseTransferApi_Sts(StsTransferApi):
    type: typing.Literal["sts"] = "sts"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


CreateAssetResponseTransferApi = typing.Union[
    CreateAssetResponseTransferApi_None, CreateAssetResponseTransferApi_PresignedUrl, CreateAssetResponseTransferApi_Sts
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .lora_tune import LoraTune
from .text_to_speech_latent_tune import TextToSpeechLatentTune


class Details_LoraTune(LoraTune):
    tune_type: typing.Literal["lora_tune"] = "lora_tune"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class Details_TextToSpeechLatentTune(TextToSpeechLatentTune):
    tune_type: typing.Literal["text_to_speech_latent_tune"] = "text_to_speech_latent_tune"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


Details = typing.Union[Details_LoraTune, Details_TextToSpeechLatentTune]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    BaseEngine,
    Details,
    Details_LoraTune,
    Details_TextToSpeechLatentTune,
    HttpValidationError,
    ListTunesResponse,
    LoraTune,
    LoraTuneCheckpoint,
    LoraTuneFile,
    TextToSpeechLatentTune,
    Tune,
    TuneDetails,
    TuneDetails_LoraTune,
    TuneDetails_TextToSpeechLatentTune,
    TuneResult,
    TuneStatus,
    TuneType,
    ValidationError,
    ValidationErrorLocItem,
)
from .errors import UnprocessableEntityError

__all__ = [
    "BaseEngine",
    "Details",
    "Details_LoraTune",
    "Details_TextToSpeechLatentTune",
    "HttpValidationError",
    "ListTunesResponse",
    "LoraTune",
    "LoraTuneCheckpoint",
    "LoraTuneFile",
    "TextToSpeechLatentTune",
    "Tune",
    "TuneDetails",
    "TuneDetails_LoraTune",
    "TuneDetails_TextToSpeechLatentTune",
    "TuneResult",
    "TuneStatus",
    "TuneType",
    "UnprocessableEntityError",
    "ValidationError",
    "ValidationErrorLocItem",
]



# This file was auto-generated by Fern from our API Definition.

from . import asset_lake, fine_tuning, image_gen, text_gen
from .environment import OctoAIEnvironment
from .version import __version__

__all__ = ["OctoAIEnvironment", "__version__", "asset_lake", "fine_tuning", "image_gen", "text_gen"]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .checkpoint_data import CheckpointData
from .file_data import FileData
from .latent_data import LatentData
from .lora_data import LoraData
from .textual_inversion_data import TextualInversionData
from .vae_data import VaeData


class Data_Checkpoint(CheckpointData):
    asset_type: typing.Literal["checkpoint"] = "checkpoint"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class Data_File(FileData):
    asset_type: typing.Literal["file"] = "file"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class Data_Latent(LatentData):
    asset_type: typing.Literal["latent"] = "latent"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class Data_Lora(LoraData):
    asset_type: typing.Literal["lora"] = "lora"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class Data_TextualInversion(TextualInversionData):
    asset_type: typing.Literal["textual_inversion"] = "textual_inversion"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class Data_Vae(VaeData):
    asset_type: typing.Literal["vae"] = "vae"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


Data = typing.Union[Data_Checkpoint, Data_File, Data_Latent, Data_Lora, Data_TextualInversion, Data_Vae]



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.remove_none_from_dict import remove_none_from_dict
from ..core.request_options import RequestOptions
from .errors.unprocessable_entity_error import UnprocessableEntityError
from .types.asset_type import AssetType
from .types.base_engine import BaseEngine
from .types.base_engine_type import BaseEngineType
from .types.complete_asset_upload_response import CompleteAssetUploadResponse
from .types.create_asset_response import CreateAssetResponse
from .types.data import Data
from .types.data_type import DataType
from .types.delete_asset_response import DeleteAssetResponse
from .types.http_validation_error import HttpValidationError
from .types.list_assets_response import ListAssetsResponse
from .types.retrieve_asset_response import RetrieveAssetResponse
from .types.transfer_api_type import TransferApiType

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AssetLakeClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        name: typing.Optional[str] = None,
        is_public: typing.Optional[bool] = None,
        data_type: typing.Optional[DataType] = None,
        asset_type: typing.Optional[typing.Union[AssetType, typing.Sequence[AssetType]]] = None,
        engine: typing.Optional[typing.Union[BaseEngine, typing.Sequence[BaseEngine]]] = None,
        engine_type: typing.Optional[BaseEngineType] = None,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        get_preview_urls: typing.Optional[bool] = None,
        asset_ids: typing.Optional[str] = None,
        owner: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListAssetsResponse:
        """
        Parameters:
            - name: typing.Optional[str].

            - is_public: typing.Optional[bool].

            - data_type: typing.Optional[DataType].

            - asset_type: typing.Optional[typing.Union[AssetType, typing.Sequence[AssetType]]].

            - engine: typing.Optional[typing.Union[BaseEngine, typing.Sequence[BaseEngine]]].

            - engine_type: typing.Optional[BaseEngineType].

            - limit: typing.Optional[int].

            - offset: typing.Optional[int].

            - get_preview_urls: typing.Optional[bool].

            - asset_ids: typing.Optional[str].

            - owner: typing.Optional[str].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import OctoAI

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.asset_lake.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_environment().default}/", "v1/assets"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "name": name,
                        "is_public": is_public,
                        "data_type": data_type,
                        "asset_type": asset_type,
                        "engine": engine,
                        "engine_type": engine_type,
                        "limit": limit,
                        "offset": offset,
                        "get_preview_urls": get_preview_urls,
                        "asset_ids": asset_ids,
                        "owner": owner,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ListAssetsResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        asset_type: AssetType,
        data: Data,
        description: typing.Optional[str] = OMIT,
        hf_repo: typing.Optional[str] = OMIT,
        hf_token_secret: typing.Optional[str] = OMIT,
        is_public: typing.Optional[bool] = OMIT,
        name: str,
        skip_validation: typing.Optional[bool] = OMIT,
        transfer_api_type: typing.Optional[TransferApiType] = OMIT,
        url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateAssetResponse:
        """
        Parameters:
            - asset_type: AssetType. Asset type.

            - data: Data. Asset data.

            - description: typing.Optional[str].

            - hf_repo: typing.Optional[str].

            - hf_token_secret: typing.Optional[str].

            - is_public: typing.Optional[bool]. True if asset is public.

            - name: str. Asset name.

            - skip_validation: typing.Optional[bool]. Skip asset validation.

            - transfer_api_type: typing.Optional[TransferApiType]. Transfer API type.

            - url: typing.Optional[str].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.asset_lake import Data_Checkpoint
        from octoai.client import OctoAI

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.asset_lake.create(
            asset_type="file",
            data=Data_Checkpoint(
                data_type="fp16",
                engine="text/llama-2-7b",
                file_format="safetensors",
            ),
            description="string",
            hf_repo="string",
            hf_token_secret="string",
            is_public=True,
            name="string",
            skip_validation=True,
            transfer_api_type="presigned-url",
            url="string",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"asset_type": asset_type, "data": data, "name": name}
        if description is not OMIT:
            _request["description"] = description
        if hf_repo is not OMIT:
            _request["hf_repo"] = hf_repo
        if hf_token_secret is not OMIT:
            _request["hf_token_secret"] = hf_token_secret
        if is_public is not OMIT:
            _request["is_public"] = is_public
        if skip_validation is not OMIT:
            _request["skip_validation"] = skip_validation
        if transfer_api_type is not OMIT:
            _request["transfer_api_type"] = transfer_api_type
        if url is not OMIT:
            _request["url"] = url
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_environment().default}/", "v1/assets"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(CreateAssetResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, asset_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> DeleteAssetResponse:
        """
        Parameters:
            - asset_id: str.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import OctoAI

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.asset_lake.delete(
            asset_id="asset_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_environment().default}/", f"v1/assets/{jsonable_encoder(asset_id)}"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(DeleteAssetResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def complete_upload(
        self,
        asset_id: str,
        *,
        token: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CompleteAssetUploadResponse:
        """
        Parameters:
            - asset_id: str.

            - token: typing.Optional[str]. Unused

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import OctoAI

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.asset_lake.complete_upload(
            asset_id="asset_id",
        )
        """
        _request: typing.Dict[str, typing.Any] = {}
        if token is not OMIT:
            _request["token"] = token
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_environment().default}/",
                f"v1/assets/{jsonable_encoder(asset_id)}/complete-upload",
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(CompleteAssetUploadResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self,
        asset_owner_and_name_or_id: str,
        *,
        transfer_api_type: typing.Optional[TransferApiType] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RetrieveAssetResponse:
        """
        Parameters:
            - asset_owner_and_name_or_id: str.

            - transfer_api_type: typing.Optional[TransferApiType].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import OctoAI

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.asset_lake.get(
            asset_owner_and_name_or_id="string",
            transfer_api_type="presigned-url",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_environment().default}/",
                f"v1/assets/{jsonable_encoder(asset_owner_and_name_or_id)}",
            ),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "transfer_api_type": transfer_api_type,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(RetrieveAssetResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncAssetLakeClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        name: typing.Optional[str] = None,
        is_public: typing.Optional[bool] = None,
        data_type: typing.Optional[DataType] = None,
        asset_type: typing.Optional[typing.Union[AssetType, typing.Sequence[AssetType]]] = None,
        engine: typing.Optional[typing.Union[BaseEngine, typing.Sequence[BaseEngine]]] = None,
        engine_type: typing.Optional[BaseEngineType] = None,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        get_preview_urls: typing.Optional[bool] = None,
        asset_ids: typing.Optional[str] = None,
        owner: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListAssetsResponse:
        """
        Parameters:
            - name: typing.Optional[str].

            - is_public: typing.Optional[bool].

            - data_type: typing.Optional[DataType].

            - asset_type: typing.Optional[typing.Union[AssetType, typing.Sequence[AssetType]]].

            - engine: typing.Optional[typing.Union[BaseEngine, typing.Sequence[BaseEngine]]].

            - engine_type: typing.Optional[BaseEngineType].

            - limit: typing.Optional[int].

            - offset: typing.Optional[int].

            - get_preview_urls: typing.Optional[bool].

            - asset_ids: typing.Optional[str].

            - owner: typing.Optional[str].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import AsyncOctoAI

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.asset_lake.list()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_environment().default}/", "v1/assets"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "name": name,
                        "is_public": is_public,
                        "data_type": data_type,
                        "asset_type": asset_type,
                        "engine": engine,
                        "engine_type": engine_type,
                        "limit": limit,
                        "offset": offset,
                        "get_preview_urls": get_preview_urls,
                        "asset_ids": asset_ids,
                        "owner": owner,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ListAssetsResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        asset_type: AssetType,
        data: Data,
        description: typing.Optional[str] = OMIT,
        hf_repo: typing.Optional[str] = OMIT,
        hf_token_secret: typing.Optional[str] = OMIT,
        is_public: typing.Optional[bool] = OMIT,
        name: str,
        skip_validation: typing.Optional[bool] = OMIT,
        transfer_api_type: typing.Optional[TransferApiType] = OMIT,
        url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateAssetResponse:
        """
        Parameters:
            - asset_type: AssetType. Asset type.

            - data: Data. Asset data.

            - description: typing.Optional[str].

            - hf_repo: typing.Optional[str].

            - hf_token_secret: typing.Optional[str].

            - is_public: typing.Optional[bool]. True if asset is public.

            - name: str. Asset name.

            - skip_validation: typing.Optional[bool]. Skip asset validation.

            - transfer_api_type: typing.Optional[TransferApiType]. Transfer API type.

            - url: typing.Optional[str].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.asset_lake import Data_Checkpoint
        from octoai.client import AsyncOctoAI

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.asset_lake.create(
            asset_type="file",
            data=Data_Checkpoint(
                data_type="fp16",
                engine="text/llama-2-7b",
                file_format="safetensors",
            ),
            description="string",
            hf_repo="string",
            hf_token_secret="string",
            is_public=True,
            name="string",
            skip_validation=True,
            transfer_api_type="presigned-url",
            url="string",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"asset_type": asset_type, "data": data, "name": name}
        if description is not OMIT:
            _request["description"] = description
        if hf_repo is not OMIT:
            _request["hf_repo"] = hf_repo
        if hf_token_secret is not OMIT:
            _request["hf_token_secret"] = hf_token_secret
        if is_public is not OMIT:
            _request["is_public"] = is_public
        if skip_validation is not OMIT:
            _request["skip_validation"] = skip_validation
        if transfer_api_type is not OMIT:
            _request["transfer_api_type"] = transfer_api_type
        if url is not OMIT:
            _request["url"] = url
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_environment().default}/", "v1/assets"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(CreateAssetResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, asset_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteAssetResponse:
        """
        Parameters:
            - asset_id: str.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import AsyncOctoAI

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.asset_lake.delete(
            asset_id="asset_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_environment().default}/", f"v1/assets/{jsonable_encoder(asset_id)}"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(DeleteAssetResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def complete_upload(
        self,
        asset_id: str,
        *,
        token: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CompleteAssetUploadResponse:
        """
        Parameters:
            - asset_id: str.

            - token: typing.Optional[str]. Unused

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import AsyncOctoAI

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.asset_lake.complete_upload(
            asset_id="asset_id",
        )
        """
        _request: typing.Dict[str, typing.Any] = {}
        if token is not OMIT:
            _request["token"] = token
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_environment().default}/",
                f"v1/assets/{jsonable_encoder(asset_id)}/complete-upload",
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(CompleteAssetUploadResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self,
        asset_owner_and_name_or_id: str,
        *,
        transfer_api_type: typing.Optional[TransferApiType] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RetrieveAssetResponse:
        """
        Parameters:
            - asset_owner_and_name_or_id: str.

            - transfer_api_type: typing.Optional[TransferApiType].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import AsyncOctoAI

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.asset_lake.get(
            asset_owner_and_name_or_id="string",
            transfer_api_type="presigned-url",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_environment().default}/",
                f"v1/assets/{jsonable_encoder(asset_owner_and_name_or_id)}",
            ),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "transfer_api_type": transfer_api_type,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(RetrieveAssetResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import pydantic_v1
from .image_encoding import ImageEncoding
from .image_generation_request_seed import ImageGenerationRequestSeed
from .scheduler import Scheduler
from .sdxl_styles import SdxlStyles


class ImageGenerationRequest(pydantic_v1.BaseModel):
    """
    Generate one or more images based on the given parameters.
    """

    prompt: str = pydantic_v1.Field()
    """
    Text describing the image content to generate.
    """

    prompt2: typing.Optional[str] = pydantic_v1.Field(alias="prompt_2", default=None)
    negative_prompt: typing.Optional[str] = None
    negative_prompt2: typing.Optional[str] = pydantic_v1.Field(alias="negative_prompt_2", default=None)
    checkpoint: typing.Optional[str] = None
    controlnet: typing.Optional[str] = None
    vae: typing.Optional[str] = None
    textual_inversions: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None
    loras: typing.Optional[typing.Dict[str, typing.Optional[float]]] = None
    sampler: typing.Optional[Scheduler] = pydantic_v1.Field(default=None)
    """
    Sampler name (also known as 'scheduler') to use during image generation.
    """

    height: typing.Optional[int] = None
    width: typing.Optional[int] = None
    cfg_scale: typing.Optional[float] = pydantic_v1.Field(default=None)
    """
    Floating-point number represeting how closely to adhere to prompt description. Must be a positive number no greater than 50.0.
    """

    steps: typing.Optional[int] = pydantic_v1.Field(default=None)
    """
    Integer repreenting how many steps of diffusion to run. Must be greater than 0 and less than or equal to 200.
    """

    num_images: typing.Optional[int] = pydantic_v1.Field(default=None)
    """
    Integer representing how many output images to generate with a single prompt/configuration.
    """

    seed: typing.Optional[ImageGenerationRequestSeed] = pydantic_v1.Field(default=None)
    """
    Integer number or list of integers representing the seeds of random generators. Fixing random seed is useful when attempting to generate a specific image. Must be greater than 0 and less than 2^32.
    """

    controlnet_image: typing.Optional[str] = None
    init_image: typing.Optional[str] = None
    mask_image: typing.Optional[str] = None
    strength: typing.Optional[float] = pydantic_v1.Field(default=None)
    """
    Floating-point number indicating how much creative the Image to Image generation mode should be. Must be greater than 0 and less than or equal to 1.0.
    """

    style_preset: typing.Optional[SdxlStyles] = None
    use_refiner: typing.Optional[bool] = pydantic_v1.Field(default=None)
    """
    Whether to enable and apply the SDXL refiner model to the image generation.
    """

    high_noise_frac: typing.Optional[float] = pydantic_v1.Field(default=None)
    """
    Floating-point number that defines the fraction of steps to perform with the base model. Used only by SD XL. Must be greater than or equal to 0.0 and less than or equal to 1.0.
    """

    controlnet_conditioning_scale: typing.Optional[float] = pydantic_v1.Field(default=None)
    """
    How strong the effect of the controlnet should be.
    """

    controlnet_early_stop: typing.Optional[float] = None
    controlnet_preprocess: typing.Optional[bool] = pydantic_v1.Field(default=None)
    """
    Whether to apply automatic ControlNet preprocessing.
    """

    clip_skip: typing.Optional[int] = None
    outpainting: typing.Optional[bool] = pydantic_v1.Field(default=None)
    """
    Whether the request requires outpainting or not. If so, special preprocessing is applied for better results.
    """

    image_encoding: typing.Optional[ImageEncoding] = pydantic_v1.Field(default=None)
    """
    Define which encoding process should be applied before returning the generated image(s).
    """

    transfer_images: typing.Optional[typing.Dict[str, typing.Optional[typing.List[str]]]] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import pydantic_v1
from .chat_completion_chunk_choice import ChatCompletionChunkChoice


class ChatCompletionChunk(pydantic_v1.BaseModel):
    """
    Represents a single chunk of a streaming chat completion response.

    This object's schema is compatible with OpenAI's Chat Completion API.
    """

    choices: typing.List[ChatCompletionChunkChoice] = pydantic_v1.Field()
    """
    A list of chat completion choices.Can be more than one if n is greater than 1.
    """

    created: int = pydantic_v1.Field()
    """
    The Unix timestamp (in seconds) of whenthe chat completion was created. Each chunk has the same timestamp.
    """

    id: str = pydantic_v1.Field()
    """
    A unique identifier for the entire chat completion request.Each chunk in the stream has the same ID.
    """

    model: str = pydantic_v1.Field()
    """
    The model used for the chat completion.
    """

    object: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    The object type, which is always `chat.completion.chunk`.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing

Scheduler = typing.Union[
    typing.Literal[
        "PNDM",
        "LMS",
        "KLMS",
        "DDIM",
        "DDPM",
        "HEUN",
        "K_HEUN",
        "K_EULER",
        "K_EULER_ANCESTRAL",
        "DPM_SOLVER_MULTISTEP",
        "DPM_PLUS_PLUS_2M_KARRAS",
        "DPM_SINGLE",
        "DPM_2",
        "DPM_2_ANCESTRAL",
        "DPM_PLUS_PLUS_SDE_KARRAS",
        "UNI_PC",
        "LCM",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .lora_tune import LoraTune
from .text_to_speech_latent_tune import TextToSpeechLatentTune


class TuneDetails_LoraTune(LoraTune):
    tune_type: typing.Literal["lora_tune"] = "lora_tune"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


class TuneDetails_TextToSpeechLatentTune(TextToSpeechLatentTune):
    tune_type: typing.Literal["text_to_speech_latent_tune"] = "text_to_speech_latent_tune"

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True


TuneDetails = typing.Union[TuneDetails_LoraTune, TuneDetails_TextToSpeechLatentTune]



# This file was auto-generated by Fern from our API Definition.

import typing

TuneType = typing.Union[
    typing.Literal[
        "lora_tune", "text_to_speech_latent_tune", "text_to_speech_rvc_tune", "text_to_speech_tortoise_tune"
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.remove_none_from_dict import remove_none_from_dict
from ..core.request_options import RequestOptions
from .errors.unprocessable_entity_error import UnprocessableEntityError
from .types.details import Details
from .types.http_validation_error import HttpValidationError
from .types.list_tunes_response import ListTunesResponse
from .types.tune import Tune
from .types.tune_type import TuneType

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class FineTuningClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        continue_on_rejection: typing.Optional[bool] = OMIT,
        description: typing.Optional[str] = OMIT,
        details: Details,
        name: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Tune:
        """
        Spawn a tune.

        Parameters:
            - continue_on_rejection: typing.Optional[bool].

            - description: typing.Optional[str].

            - details: Details. Details of the tune.

            - name: str. The name of the tune.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import OctoAI
        from octoai.fine_tuning import (
            Details_LoraTune,
            LoraTuneCheckpoint,
            LoraTuneFile,
        )

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.fine_tuning.create(
            continue_on_rejection=True,
            description="string",
            details=Details_LoraTune(
                base_checkpoint=LoraTuneCheckpoint(),
                files=[LoraTuneFile()],
                resize_images=True,
                seed=1,
                steps=1,
                trigger_words=["string"],
            ),
            name="string",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"details": details, "name": name}
        if continue_on_rejection is not OMIT:
            _request["continue_on_rejection"] = continue_on_rejection
        if description is not OMIT:
            _request["description"] = description
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_environment().default}/", "v1/tune"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Tune, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, tune_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Tune:
        """
        Get the specific tune.

        Parameters:
            - tune_id: str. The ID of the tune.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import OctoAI

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.fine_tuning.get(
            tune_id="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_environment().default}/", f"v1/tune/{jsonable_encoder(tune_id)}"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Tune, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self, tune_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Any]:
        """
        Delete the specified tune.

        Parameters:
            - tune_id: str. The ID of the tune.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import OctoAI

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.fine_tuning.delete(
            tune_id="tune_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_environment().default}/", f"v1/tune/{jsonable_encoder(tune_id)}"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(typing.Dict[str, typing.Any], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def cancel(self, tune_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Tune:
        """
        Cancel the specified tune.

        Parameters:
            - tune_id: str. The ID of the tune to cancel.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import OctoAI

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.fine_tuning.cancel(
            tune_id="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_environment().default}/", f"v1/tune/{jsonable_encoder(tune_id)}/cancel"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))
            if request_options is not None
            else None,
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Tune, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def list(
        self,
        *,
        offset: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        name: typing.Optional[str] = None,
        tune_type: typing.Optional[TuneType] = None,
        base_checkpoint_id: typing.Optional[str] = None,
        trigger_words: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        engine: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListTunesResponse:
        """
        List all tunes owned by the current user.

        Parameters:
            - offset: typing.Optional[int]. Offset into the results.

            - limit: typing.Optional[int]. The max number of results to be shown (limit 100).

            - name: typing.Optional[str]. The name of the tune to filter on.

            - tune_type: typing.Optional[TuneType]. The type of the tune to filter on.

            - base_checkpoint_id: typing.Optional[str]. The base checkpoint ID for the LoRA tune.

            - trigger_words: typing.Optional[typing.Union[str, typing.Sequence[str]]]. The trigger words to filter on.

            - engine: typing.Optional[str]. The engine type.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import OctoAI

        client = OctoAI(
            api_key="YOUR_API_KEY",
        )
        client.fine_tuning.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_environment().default}/", "v1/tunes"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "offset": offset,
                        "limit": limit,
                        "name": name,
                        "tune_type": tune_type,
                        "base_checkpoint_id": base_checkpoint_id,
                        "trigger_words": trigger_words,
                        "engine": engine,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ListTunesResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncFineTuningClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        continue_on_rejection: typing.Optional[bool] = OMIT,
        description: typing.Optional[str] = OMIT,
        details: Details,
        name: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Tune:
        """
        Spawn a tune.

        Parameters:
            - continue_on_rejection: typing.Optional[bool].

            - description: typing.Optional[str].

            - details: Details. Details of the tune.

            - name: str. The name of the tune.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import AsyncOctoAI
        from octoai.fine_tuning import (
            Details_LoraTune,
            LoraTuneCheckpoint,
            LoraTuneFile,
        )

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.fine_tuning.create(
            continue_on_rejection=True,
            description="string",
            details=Details_LoraTune(
                base_checkpoint=LoraTuneCheckpoint(),
                files=[LoraTuneFile()],
                resize_images=True,
                seed=1,
                steps=1,
                trigger_words=["string"],
            ),
            name="string",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"details": details, "name": name}
        if continue_on_rejection is not OMIT:
            _request["continue_on_rejection"] = continue_on_rejection
        if description is not OMIT:
            _request["description"] = description
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_environment().default}/", "v1/tune"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Tune, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, tune_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Tune:
        """
        Get the specific tune.

        Parameters:
            - tune_id: str. The ID of the tune.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import AsyncOctoAI

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.fine_tuning.get(
            tune_id="string",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_environment().default}/", f"v1/tune/{jsonable_encoder(tune_id)}"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Tune, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, tune_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Any]:
        """
        Delete the specified tune.

        Parameters:
            - tune_id: str. The ID of the tune.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import AsyncOctoAI

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.fine_tuning.delete(
            tune_id="tune_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_environment().default}/", f"v1/tune/{jsonable_encoder(tune_id)}"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(typing.Dict[str, typing.Any], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def cancel(self, tune_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Tune:
        """
        Cancel the specified tune.

        Parameters:
            - tune_id: str. The ID of the tune to cancel.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import AsyncOctoAI

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.fine_tuning.cancel(
            tune_id="string",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_environment().default}/", f"v1/tune/{jsonable_encoder(tune_id)}/cancel"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))
            if request_options is not None
            else None,
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Tune, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def list(
        self,
        *,
        offset: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        name: typing.Optional[str] = None,
        tune_type: typing.Optional[TuneType] = None,
        base_checkpoint_id: typing.Optional[str] = None,
        trigger_words: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        engine: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListTunesResponse:
        """
        List all tunes owned by the current user.

        Parameters:
            - offset: typing.Optional[int]. Offset into the results.

            - limit: typing.Optional[int]. The max number of results to be shown (limit 100).

            - name: typing.Optional[str]. The name of the tune to filter on.

            - tune_type: typing.Optional[TuneType]. The type of the tune to filter on.

            - base_checkpoint_id: typing.Optional[str]. The base checkpoint ID for the LoRA tune.

            - trigger_words: typing.Optional[typing.Union[str, typing.Sequence[str]]]. The trigger words to filter on.

            - engine: typing.Optional[str]. The engine type.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from octoai.client import AsyncOctoAI

        client = AsyncOctoAI(
            api_key="YOUR_API_KEY",
        )
        await client.fine_tuning.list()
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_environment().default}/", "v1/tunes"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "offset": offset,
                        "limit": limit,
                        "name": name,
                        "tune_type": tune_type,
                        "base_checkpoint_id": base_checkpoint_id,
                        "trigger_words": trigger_words,
                        "engine": engine,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ListTunesResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import pydantic_v1
from .asset import Asset
from .create_asset_response_transfer_api import CreateAssetResponseTransferApi


class CreateAssetResponse(pydantic_v1.BaseModel):
    asset: Asset = pydantic_v1.Field()
    """
    Asset.
    """

    token: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Unused. Present for compatibility purposes.
    """

    transfer_api: CreateAssetResponseTransferApi = pydantic_v1.Field()
    """
    Asset transfer API.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import pydantic_v1
from .asset_type import AssetType
from .data import Data
from .file_structure import FileStructure
from .status import Status


class Asset(pydantic_v1.BaseModel):
    asset_type: AssetType = pydantic_v1.Field()
    """
    Asset type.
    """

    created_at: str = pydantic_v1.Field()
    """
    Asset creation time.
    """

    data: Data = pydantic_v1.Field()
    """
    Asset data.
    """

    description: str = pydantic_v1.Field()
    """
    Description.
    """

    file_structure: typing.Optional[FileStructure] = pydantic_v1.Field(default=None)
    """
    File structure.
    """

    hf_repo: typing.Optional[str] = None
    id: str = pydantic_v1.Field()
    """
    Asset id.
    """

    is_public: typing.Optional[bool] = pydantic_v1.Field(default=None)
    """
    True if asset is public.
    """

    name: str = pydantic_v1.Field()
    """
    Asset name.
    """

    size_bytes: int = pydantic_v1.Field()
    """
    Size in bytes.
    """

    status: Status = pydantic_v1.Field()
    """
    Asset status.
    """

    status_details: str = pydantic_v1.Field()
    """
    Asset status.
    """

    status_info: typing.Optional[typing.Dict[str, typing.Any]] = pydantic_v1.Field(default=None)
    """
    Asset status info.
    """

    tenant_uuid: str = pydantic_v1.Field()
    """
    Asset tenant ID.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing

SdxlStyles = typing.Union[
    typing.Literal[
        "base",
        "3d-model",
        "analog-film",
        "anime",
        "cinematic",
        "comic-book",
        "Craft Clay",
        "modeling-compound",
        "digital-art",
        "enhance",
        "fantasy-art",
        "isometric",
        "line-art",
        "low-poly",
        "neon-punk",
        "origami",
        "photographic",
        "pixel-art",
        "tile-texture",
        "Advertising",
        "Food Photography",
        "Real Estate",
        "Abstract",
        "Cubist",
        "Graffiti",
        "Hyperrealism",
        "Impressionist",
        "Pointillism",
        "Pop Art",
        "Psychedelic",
        "Renaissance",
        "Steampunk",
        "Surrealist",
        "Typography",
        "Watercolor",
        "Fighting Game",
        "GTA",
        "Super Mario",
        "Minecraft",
        "Pokémon",
        "Retro Arcade",
        "Retro Game",
        "RPG Fantasy Game",
        "Strategy Game",
        "Street Fighter",
        "Legend of Zelda",
        "Architectural",
        "Disco",
        "Dreamscape",
        "Dystopian",
        "Fairy Tale",
        "Gothic",
        "Grunge",
        "Horror",
        "Minimalist",
        "Monochrome",
        "Nautical",
        "Space",
        "Stained Glass",
        "Techwear Fashion",
        "Tribal",
        "Zentangle",
        "Collage",
        "Flat Papercut",
        "Kirigami",
        "Paper Mache",
        "Paper Quilling",
        "Papercut Collage",
        "Papercut Shadow Box",
        "Stacked Papercut",
        "Thick Layered Papercut",
        "Alien",
        "Film Noir",
        "HDR",
        "Long Exposure",
        "Neon Noir",
        "Silhouette",
        "Tilt-Shift",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    ChatCompletionChoice,
    ChatCompletionChunk,
    ChatCompletionChunkChoice,
    ChatCompletionDelta,
    ChatCompletionRequestExt,
    ChatCompletionRequestExtVllm,
    ChatCompletionResponse,
    ChatCompletionResponseFormat,
    ChatFnCall,
    ChatMessage,
    CompletionChoice,
    CompletionResponse,
    CreateChatCompletionRequestFunctionCall,
    CreateChatCompletionStreamRequestFunctionCall,
    ErrorResponse,
    ErrorResponseValidationErrorsValue,
    FinishReason,
    Function,
    HttpValidationError,
    LogProbs,
    Prompt,
    Stop,
    UsageStats,
    ValidationError,
    ValidationErrorLocItem,
)
from .errors import InternalServerError, UnprocessableEntityError

__all__ = [
    "ChatCompletionChoice",
    "ChatCompletionChunk",
    "ChatCompletionChunkChoice",
    "ChatCompletionDelta",
    "ChatCompletionRequestExt",
    "ChatCompletionRequestExtVllm",
    "ChatCompletionResponse",
    "ChatCompletionResponseFormat",
    "ChatFnCall",
    "ChatMessage",
    "CompletionChoice",
    "CompletionResponse",
    "CreateChatCompletionRequestFunctionCall",
    "CreateChatCompletionStreamRequestFunctionCall",
    "ErrorResponse",
    "ErrorResponseValidationErrorsValue",
    "FinishReason",
    "Function",
    "HttpValidationError",
    "InternalServerError",
    "LogProbs",
    "Prompt",
    "Stop",
    "UnprocessableEntityError",
    "UsageStats",
    "ValidationError",
    "ValidationErrorLocItem",
]
